@@ -3,6 +3,8 @@
 # 2009-09-29 by J. Fox (renamed)
 # 2010-03-11 by J. Fox: output changed
 # 2010-03-13 by J. Fox: output row label fixed when just one X
+# 2022-10-25 by J. Fox: use t-distribution for score tests, add omnibus score test,
+#                       suggestions of Peter Dalgaard
 #-------------------------------------------------------------------------------
 
 
@@ -41,9 +43,17 @@ boxTidwell.default <- function(y, x1, x2=NULL, max.iter=25, tol=.001, verbose=FA
 	mod.2 <- lm(y ~ cbind(x.log.x, x1, x2), ...)
 	seb <- sqrt(diag(vcov(mod.2)))
 	which.coefs <- 2:(1 + k.x1)
+	F <- if (k.x1 > 1){
+  	H <- cbind(0, diag(k.x1), matrix(0, k.x1, k.x1))
+  	if (!is.null(x2)) H <- cbind(H, matrix(0, k.x1, ncol(x2)))
+  	linearHypothesis(mod.2, H)
+	} else {
+	  NULL
+	}
 	t.vals <- ((coefficients(mod.2))/seb)[which.coefs]
-	initial <- powers <- 1 + coefficients(mod.2)[which.coefs]/coefficients(mod.1)[which.coefs]
-	pvalues<-2*(pnorm(abs(t.vals), lower.tail=FALSE))
+	# initial <- 
+	powers <- 1 + coefficients(mod.2)[which.coefs]/coefficients(mod.1)[which.coefs]
+	pvalues <- 2*(pt(abs(t.vals), df=df.residual(mod.2), lower.tail=FALSE))
 	iter <- 0
 	last.powers <- 1
 	while ((max(abs((powers - last.powers)/(powers + tol))) > tol) && (iter <= max.iter) ) {
@@ -58,9 +68,9 @@ boxTidwell.default <- function(y, x1, x2=NULL, max.iter=25, tol=.001, verbose=FA
 	}
 	if (iter > max.iter) warning("maximum iterations exceeded")
 	result <- cbind(powers, t.vals, pvalues)
-	colnames(result) <- c("MLE of lambda", "Score Statistic (z)", "Pr(>|z|)")
+	colnames(result) <- c("MLE of lambda", "Score Statistic (t)", "Pr(>|t|)")
 	rownames(result) <- if (nrow(result) == 1) "" else var.names
-	result <- list(result=result, iterations=iter)
+	result <- list(result=result, iterations=iter, F=F)
 	class(result)<-"boxTidwell"
 	result
 }
@@ -68,5 +78,11 @@ boxTidwell.default <- function(y, x1, x2=NULL, max.iter=25, tol=.001, verbose=FA
 print.boxTidwell <- function(x, digits=getOption("digits") - 2, ...){ 
 	printCoefmat(x$result, digits=digits, has.Pvalue=TRUE)
 	cat("\niterations = ", x$iterations,"\n")
+	if (!is.null(x$F)){
+  	cat("\nScore test for null hypothesis that all lambdas = 1:\n")
+  	cat(paste0("F = ", format(x$F$F[2], digits=digits), ", df = ", x$F$Df[2], " and ", 
+  	           x$F$Res.Df[2], ", Pr(>F) = ", format.pval(x$F$"Pr(>F)"[2], digits=digits - 1), '\n\n'))
+	}
+	return(invisible(x))
 }
 
